@@ -59,6 +59,12 @@ linRegClass <- R6::R6Class(
 
             return(private$.cooks)
         },
+        mahal = function() {
+            if (is.null(private$.mahal))
+                private$.mahal <- private$.computeMahal()
+
+            return(private$.mahal)
+        },
         anovaModelComparison = function() {
             if (is.null(private$.anovaModelComparison))
                 private$.anovaModelComparison <- do.call(stats::anova, self$models)
@@ -112,6 +118,19 @@ linRegClass <- R6::R6Class(
                 private$.emMeans <- private$.computeEmMeans()
 
             return(private$.emMeans)
+        },
+        refLevels = function() {
+            if (is.null(private$.refLevels)) {
+                refLevels <- getReferenceLevels(
+                    self$data, self$options$factors, self$options$refLevels
+                )
+                private$.refLevels <- refLevels$refLevels
+
+                if (length(refLevels$changedVars) > 0)
+                    setRefLevelWarning(self, refLevels$changedVars)
+            }
+
+            return(private$.refLevels)
         }
     ),
     private = list(
@@ -127,6 +146,7 @@ linRegClass <- R6::R6Class(
         .fitted = NULL,
         .predicted = NULL,
         .cooks = NULL,
+        .mahal = NULL,
         .anovaModelComparison = NULL,
         .anovaModelTerms = NULL,
         .AIC = NULL,
@@ -139,6 +159,7 @@ linRegClass <- R6::R6Class(
         .rowNamesModel = NULL,
         .emMeans = NULL,
         .emMeansForPlot = NULL,
+        .refLevels = NULL,
 
         #### Init + run functions ----
         .init = function() {
@@ -147,6 +168,7 @@ linRegClass <- R6::R6Class(
             private$.initModelSpec()
             private$.initAnovaTables()
             private$.initCoefTable()
+            private$.initMahalTable()
             private$.initCollinearityTable()
             private$.initResPlots()
             private$.initEmm()
@@ -157,12 +179,16 @@ linRegClass <- R6::R6Class(
             if (is.null(self$options$dep) || self$nModels < 1 || length(self$options$blocks[[1]]) == 0)
                 return()
 
+            if (any(unlist(private$.getIsAliased())))
+                setSingularityWarning(self)
+
             private$.populateModelFitTable()
             private$.populateModelCompTable()
             private$.populateAnovaTables()
             private$.populateCoefTables()
 
             private$.populateCooksTable()
+            private$.populateMahalTable()
             private$.populateDurbinWatsonTable()
             private$.populateCollinearityTable()
             private$.populateNormality()
@@ -271,6 +297,27 @@ linRegClass <- R6::R6Class(
 
             return(cooksSummary)
         },
+        .computeMahal = function() {
+            mahal <- vector(mode = "list", length(self$nModels))
+            data <- self$dataProcessed
+            for (i in seq_along(self$models)) {
+                terms <- attr(self$models[[i]]$terms, "term.labels")
+                covs <- intersect(terms, jmvcore::toB64(self$options$covs))
+                # skip if number of variables less than 2
+                if (length(covs) >= 2) {
+                    mahalChiSq <- stats::mahalanobis(
+                        data[, covs], colMeans(data[, covs]), cov(data[, covs])
+                    )
+                    mahal[[i]] <- data.frame(
+                        chisq = mahalChiSq,
+                        p = pchisq(mahalChiSq, length(covs), lower.tail = FALSE),
+                        row.names = rownames(data)
+                    )
+                }
+            }
+
+            return(mahal)
+        },
         .computeAIC = function() {
             AIC <- list()
             for (i in seq_along(self$models))
@@ -302,10 +349,11 @@ linRegClass <- R6::R6Class(
 
             VIF <- list()
             for (i in seq_along(self$models)) {
-                if ( ! private$.getIsAliased()[[i]] && length(modelTerms[[i]]) > 1 )
+                if ( ! private$.getIsAliased()[[i]] && length(modelTerms[[i]]) > 1 ) {
                     VIF[[i]] <- car::vif(self$models[[i]])
-                else
-                    VIF[[i]] <- NULL
+                } else {
+                    VIF[[i]] <- c(VIF, list(NULL))
+                }
             }
 
             return(VIF)
@@ -498,6 +546,37 @@ linRegClass <- R6::R6Class(
                 }
             }
         },
+        .initMahalTable = function() {
+            if ( ! self$options$mahal)
+                return()
+
+            groups <- self$results$models
+            termsAll <- private$.getModelTerms()
+            covs <- self$options$covs
+
+            for (i in seq_along(termsAll)) {
+                modelTerms <- termsAll[[i]]
+                modelCovs <- intersect(modelTerms, covs)
+
+                group <- groups[[i]]$dataSummary
+
+                if (length(modelCovs) == 1) {
+                    mahalNote <- jmvcore::Notice$new(
+                        options = self$options,
+                        name = 'warningMessage',
+                        type = jmvcore::NoticeType$WARNING
+                    )
+                    mahalNote$setContent(
+                        .("Mahalanobis distance can only be calculated for models with two or more covariates")
+                    )
+
+                    group$insert(1, mahalNote)
+                    group$mahal$setVisible(FALSE)
+                } else {
+                    group$mahal$setVisible(TRUE)
+                }
+            }
+        },
         .initCollinearityTable = function() {
             groups <- self$results$models
             termsAll <- private$.getModelTerms()
@@ -639,6 +718,28 @@ linRegClass <- R6::R6Class(
                 titles <- vapply(keys, function(key) title(.("Cook's distance"), key), '')
                 descriptions <- vapply(keys, function(key) description(.("Cook's distance"), key), '')
                 self$results$cooksOV$set(keys, titles, descriptions, measureTypes)
+
+                titles <- c(
+                    vapply(keys, function(key) title(.("Mahalanobis ChiSq"), key), ''),
+                    vapply(keys, function(key) title(.("Mahalanobis p"),     key), '')
+                )
+                descriptions <- c(
+                    vapply(keys, function(key) description(.("Mahalanobis distance (Chi sq.)"), key), ''),
+                    vapply(keys, function(key) description(.("Mahalanobis distance (p)"),       key), '')
+                )
+                order <- as.vector(
+                    vapply(seq(self$nModels), function(p) c(p, p + self$nModels), numeric(2))
+                )
+                self$results$mahalOV$set(
+                    seq(2 * self$nModels), titles[order], descriptions[order], rep(measureTypes, 2)
+                )
+            } else {
+                titles <- c(.("Mahalanobis ChiSq"), .("Mahalanobis p"))
+                descriptions <- c(
+                    description(.("Mahalanobis distance (Chi sq.)")),
+                    description(.("Mahalanobis distance (p)"))
+                )
+                self$results$mahalOV$set(seq(2), titles, descriptions, rep('continuous', 2))
             }
         },
 
@@ -836,6 +937,39 @@ linRegClass <- R6::R6Class(
                 table$setRow(rowNo=1, values=cooks[[i]])
             }
         },
+        .populateMahalTable = function() {
+            if ( ! self$options$mahal)
+                return()
+
+            mahal <- self$mahal
+            mahalp <- as.numeric(self$options$mahalp)
+            groups <- self$results$models
+            termsAll <- private$.getModelTerms()
+
+            for (i in seq_along(termsAll)) {
+                table <- groups$get(key=i)$dataSummary$mahal
+                if (i <= length(mahal) && ! is.null(mahal[[i]])) {
+                    mahalRow <- rownames(mahal[[i]][mahal[[i]]$p <= mahalp, ])
+                    mahalVal = list(
+                        mean = mean(mahal[[i]]$chisq),
+                        median = median(mahal[[i]]$chisq),
+                        sd = sd(mahal[[i]]$chisq),
+                        min = min(mahal[[i]]$chisq),
+                        max = max(mahal[[i]]$chisq),
+                        excRow = paste(mahalRow, collapse = ", ")
+                    )
+                    table$setRow(rowNo = 1, values = mahalVal)
+                    if (length(mahalRow) == 0) {
+                        table$setNote(
+                            "excRow",
+                            jmvcore::format(
+                                .("There were no Mahalanobis distances with a threshold below p < {p}."),
+                                p = mahalp)
+                            )
+                    }
+                }
+            }
+        },
         .populateDurbinWatsonTable = function() {
             if (! self$options$durbin)
                 return()
@@ -1006,6 +1140,15 @@ linRegClass <- R6::R6Class(
                 for (i in seq_along(self$cooks))
                     self$results$cooksOV$setValues(index=i, self$cooks[[i]])
             }
+
+            if (self$options$mahalOV && self$results$mahalOV$isNotFilled()) {
+                self$results$mahalOV$setRowNums(private$.getDataRowNums())
+                for (i in seq_along(self$mahal)) {
+                    if (is.null(self$mahal[[i]])) next
+                    self$results$mahalOV$setValues(index=(i * 2 - 1), self$mahal[[i]]$chisq)
+                    self$results$mahalOV$setValues(index=(i * 2 - 0), self$mahal[[i]]$p)
+                }
+            }
         },
 
         #### Plot functions ----
@@ -1284,7 +1427,7 @@ linRegClass <- R6::R6Class(
             #' displayed in the coef table
 
             factors <- self$options$factors
-            refLevels <- self$options$refLevels
+            refLevels <- self$refLevels
             refVars <- sapply(refLevels, function(x) x$var)
 
             levels <- list()
@@ -1333,7 +1476,7 @@ linRegClass <- R6::R6Class(
             covs <- self$options$covs
             factors <- self$options$factors
             weights <- self$options$weights
-            refLevels <- self$options$refLevels
+            refLevels <- self$refLevels
 
             dataRaw <- self$data
 
