@@ -31,7 +31,11 @@ testthat::test_that('All options in the contTables work (sunny)', {
         exp = TRUE,
         pcRow = TRUE,
         pcCol = TRUE,
-        pcTot = TRUE
+        pcTot = TRUE,
+        resU = TRUE,
+        resP = TRUE,
+        resS = TRUE,
+        resA = TRUE
     )
 
     # Test main contingency tables
@@ -62,6 +66,21 @@ testthat::test_that('All options in the contTables work (sunny)', {
     testthat::expect_equal(c(0.45, 0.55, 1), mainTable[['.total[pcCol]']], tolerance = 1e-3)
     testthat::expect_equal(c(0.45, 0.55, 1), mainTable[['.total[pcTot]']], tolerance = 1e-3)
 
+    # Test residuals postHoc tables
+    postHoc <- r$postHoc$asDF
+    testthat::expect_equal(c('Unstandardized residuals', 'Unstandardized residuals'), postHoc[['type[resU]']])
+    testthat::expect_equal(c('Pearson residuals', 'Pearson residuals'), postHoc[['type[resP]']])
+    testthat::expect_equal(c('Standardized residuals', 'Standardized residuals'), postHoc[['type[resS]']])
+    testthat::expect_equal(c('Deviance residuals', 'Deviance residuals'), postHoc[['type[resA]']])
+    testthat::expect_equal(c(0.0500, -0.0500), postHoc[['1[resU]']], tolerance = 1e-3)
+    testthat::expect_equal(c(-0.0500, 0.0500), postHoc[['2[resU]']], tolerance = 1e-3)
+    testthat::expect_equal(c(0.0104, -0.0094), postHoc[['1[resP]']], tolerance = 1e-3)
+    testthat::expect_equal(c(-0.0106, 0.0096), postHoc[['2[resP]']], tolerance = 1e-3)
+    testthat::expect_equal(c(0.0201, -0.0201), postHoc[['1[resS]']], tolerance = 1e-3)
+    testthat::expect_equal(c(-0.0201, 0.0201), postHoc[['2[resS]']], tolerance = 1e-3)
+    testthat::expect_equal(c(0.0104, -0.0094), postHoc[['1[resA]']], tolerance = 1e-3)
+    testthat::expect_equal(c(-0.0107, 0.0096), postHoc[['2[resA]']], tolerance = 1e-3)
+
     # Test chi squared tests table
     chiSqTable <- r$chiSq$asDF
     testthat::expect_equal('χ²', chiSqTable[['test[chiSq]']])
@@ -182,7 +201,7 @@ testthat::test_that("conttables works with counts", {
 
     data <- data.frame(rows = rows, cols = cols, layer = layer, counts = counts)
 
-    table <- jmv::contTables(data=data, rows="rows", cols="cols", layers="layer", counts="counts")
+    table <- jmv::contTables(data=data, rows="rows", cols="cols", layers="layer", counts="counts", resU=TRUE, resP=TRUE, resS=TRUE, resA=TRUE)
 
     freqs <- as.data.frame(table$freqs)
 
@@ -192,6 +211,18 @@ testthat::test_that("conttables works with counts", {
     testthat::expect_equal(0, freqs[2, '2[count]'])
     testthat::expect_equal(84, freqs[12, '1[count]'])
     testthat::expect_equal(32, freqs[12, '2[count]'])
+
+    # Test residuals postHoc tables
+    postHoc <- as.data.frame(table$postHoc)
+
+    testthat::expect_equal('Unstandardized residuals', postHoc[4, 'type[resU]'])
+    testthat::expect_equal('Pearson residuals', postHoc[6, 'type[resP]'])
+    testthat::expect_equal('Standardized residuals', postHoc[1, 'type[resS]'])
+    testthat::expect_equal('Deviance residuals', postHoc[2, 'type[resA]'])
+    testthat::expect_equal(2.111, postHoc[4, '1[resU]'], tolerance = 1e-3)
+    testthat::expect_equal(1.113, postHoc[6, '2[resP]'], tolerance = 1e-3)
+    testthat::expect_equal(-2.422, postHoc[1, '1[resS]'], tolerance = 1e-3)
+    testthat::expect_equal(-1.758, postHoc[2, '2[resA]'], tolerance = 1e-3)
 })
 
 testthat::test_that("conttables works with global integer weights", {
