@@ -75,6 +75,32 @@ testthat::test_that('All options in the logRegOrd work (sunny)', {
     testthat::expect_equal(c(0.401, 1.291), thresTable[['odds']], tolerance = 1e-3)
 })
 
+testthat::test_that("Model comparison works", {
+    # GIVEN a dataset with a dependent variable and two covariates
+    suppressWarnings(RNGversion("3.5.0"))
+    set.seed(1337)
+    df <- data.frame(
+        dep = sample(letters[1:3], 100, replace = TRUE),
+        cov1 = rnorm(100),
+        cov2 = rnorm(100),
+        stringsAsFactors = TRUE
+    )
+
+    # WHEN an ordinal regression model is fitted in two blocks
+    r <- jmv::logRegOrd(
+        data = df,
+        dep = "dep",
+        covs = c("cov1", "cov2"),
+        blocks = list(list("cov1"), list("cov2"))
+    )
+
+    # THEN the model comparison table contains the model fit statistics
+    modelCompTable <- r$modelComp$asDF
+    testthat::expect_equal(0.0369, modelCompTable$chi, tolerance = 1e-3)
+    testthat::expect_equal(1, modelCompTable$df)
+    testthat::expect_equal(0.848, modelCompTable$p, tolerance = 1e-3)
+})
+
 testthat::test_that("Analysis works with global weights", {
     suppressWarnings(RNGversion("3.5.0"))
     set.seed(1337)
@@ -134,4 +160,33 @@ testthat::test_that('Model fit table contains sample size footnote', {
     testthat::expect_match(r$modelFit$notes$n$note, "N=15")
 })
 
+params <- list(
+    list(refLevels = list(list(var="factor", ref="X")), info = "Non-existing reference levels"),
+    list(refLevels = NULL, info = "No reference levels"),
+    list(refLevels = list(list(var="wrong_factor", ref="A")), info = "Wrong variable name")
+)
+testthat::test_that('Reference level defaults to first level for faulty reference levels', {
+    for (param in params) {
+        # GIVEN a dataset with a factor with two levels
+        df <- data.frame(
+            dep = rep(letters[1:3], length.out=10),
+            factor = rep(LETTERS[1:2], length.out=10),
+            stringsAsFactors = TRUE
+        )
 
+        # WHEN a ordinal logistic regression is fitted with reference level set to a non-existing level
+        r <- jmv::logRegOrd(
+            df,
+            dep = "dep",
+            factors = "factor",
+            blocks = list(list("factor")),
+            refLevels = param$refLevels
+        )
+
+        # THEN the reference level should default to the first level
+        testthat::expect_match(r$models[[1]]$coef$asDF$term[2], "B – A", info=param$info)
+        # AND a warning is added informing the user that the user defined reference level does not
+        #   exist and therefore was changed to the first level
+        testthat::expect_match(r[[1]]$content, "reference level was not found", info=param$info)
+    }
+})
